@@ -10,6 +10,7 @@ import textwrap
 
 from .deprecation import ImplicitDeprecated, resolve_deprecate_info
 from .log import get_logger
+from .preview import ImplicitPreviewItem, resolve_preview_info
 from .util import CtxTypeError
 from .help_files import _load_help_file
 
@@ -21,13 +22,6 @@ FIRST_LINE_PREFIX = ' : '
 REQUIRED_TAG = '[Required]'
 
 
-def _get_preview_tag():
-    import colorama
-    PREVIEW_TAG = colorama.Fore.CYAN + '[Preview]' + colorama.Fore.RESET
-    PREVIEW_TAG_LEN = len(PREVIEW_TAG) - 2 * len(colorama.Fore.RESET)
-    return (PREVIEW_TAG, PREVIEW_TAG_LEN)
-
-
 def _get_hanging_indent(max_length, indent):
     return max_length + (indent * 4) + len(FIRST_LINE_PREFIX) - 1
 
@@ -44,13 +38,13 @@ def _get_line_len(name, tags_len):
     return len(name) + tags_len + (2 if tags_len else 1)
 
 
-def _print_indent(s, indent=0, subsequent_spaces=-1):
+def _print_indent(s, indent=0, subsequent_spaces=-1, width=100):
     tw = textwrap.TextWrapper(initial_indent='    ' * indent,
                               subsequent_indent=('    ' * indent
                                                  if subsequent_spaces == -1
                                                  else ' ' * subsequent_spaces),
                               replace_whitespace=False,
-                              width=100)
+                              width=width)
     paragraphs = s.split('\n')
     for p in paragraphs:
         try:
@@ -123,7 +117,7 @@ class HelpFile(HelpObject):
     def _load_help_file_from_string(text):
         import yaml
         try:
-            return yaml.load(text) if text else None
+            return yaml.safe_load(text) if text else None
         except Exception:  # pylint: disable=broad-except
             return text
 
@@ -159,6 +153,26 @@ class HelpFile(HelpObject):
             del deprecate_kwargs['_get_message']
             self.deprecate_info = ImplicitDeprecated(**deprecate_kwargs)
 
+        # resolve preview info
+        direct_preview_info = resolve_preview_info(help_ctx.cli_ctx, delimiters)
+        if direct_preview_info:
+            self.preview_info = direct_preview_info
+
+        # search for implicit preview
+        path_comps = delimiters.split()[:-1]
+        implicit_preview_info = None
+        while path_comps and not implicit_preview_info:
+            implicit_preview_info = resolve_preview_info(help_ctx.cli_ctx, ' '.join(path_comps))
+            del path_comps[-1]
+
+        if implicit_preview_info:
+            preview_kwargs = implicit_preview_info.__dict__.copy()
+            if delimiters in help_ctx.cli_ctx.invocation.commands_loader.command_table:
+                preview_kwargs['object_type'] = 'command'
+            else:
+                preview_kwargs['object_type'] = 'command group'
+            self.preview_info = ImplicitPreviewItem(**preview_kwargs)
+
     def load(self, options):
         description = getattr(options, 'description', None)
         try:
@@ -208,7 +222,6 @@ class GroupHelpFile(HelpFile):
 
         super(GroupHelpFile, self).__init__(help_ctx, delimiters)
         self.type = 'group'
-        self.preview_info = getattr(parser, 'preview_info', None)
 
         self.children = []
         if getattr(parser, 'choices', None):
@@ -237,7 +250,21 @@ class CommandHelpFile(HelpFile):
         self.parameters = []
 
         for action in [a for a in parser._actions if a.help != argparse.SUPPRESS]:  # pylint: disable=protected-access
-            self._add_parameter_help(action)
+            if action.option_strings:
+                self._add_parameter_help(action)
+            else:
+                # use metavar for positional parameters
+                param_kwargs = {
+                    'name_source': [action.metavar or action.dest],
+                    'deprecate_info': getattr(action, 'deprecate_info', None),
+                    'preview_info': getattr(action, 'preview_info', None),
+                    'description': action.help,
+                    'choices': action.choices,
+                    'required': False,
+                    'default': None,
+                    'group_name': 'Positional'
+                }
+                self.parameters.append(HelpParameter(**param_kwargs))
 
         help_param = next(p for p in self.parameters if p.name == '--help -h')
         help_param.group_name = 'Global Arguments'
@@ -267,7 +294,8 @@ class CommandHelpFile(HelpFile):
             self.parameters.append(HelpParameter(**param_kwargs))
         param_kwargs.update({
             'name_source': normal_options,
-            'deprecate_info': getattr(param, 'deprecate_info', None)
+            'deprecate_info': getattr(param, 'deprecate_info', None),
+            'preview_info': getattr(param, 'preview_info', None)
         })
         self.parameters.append(HelpParameter(**param_kwargs))
 
@@ -291,7 +319,7 @@ class CommandHelpFile(HelpFile):
 class HelpParameter(HelpObject):  # pylint: disable=too-many-instance-attributes
 
     def __init__(self, name_source, description, required, choices=None,
-                 default=None, group_name=None, deprecate_info=None):
+                 default=None, group_name=None, deprecate_info=None, preview_info=None):
         super(HelpParameter, self).__init__()
         self.name_source = name_source
         self.name = ' '.join(sorted(name_source))
@@ -304,10 +332,11 @@ class HelpParameter(HelpObject):  # pylint: disable=too-many-instance-attributes
         self.default = default
         self.group_name = group_name
         self.deprecate_info = deprecate_info
+        self.preview_info = preview_info
 
     def update_from_data(self, data):
         if self.name != data.get('name'):
-            raise HelpAuthoringException(u"mismatched name {0} vs. {1}"
+            raise HelpAuthoringException(u"mismatched name {} vs. {}"
                                          .format(self.name,
                                                  data.get('name')))
 
@@ -333,7 +362,6 @@ class HelpExample(object):  # pylint: disable=too-few-public-methods
 
 class CLIHelp(object):
 
-    # pylint: disable=no-self-use
     def _print_header(self, cli_name, help_file):
         indent = 0
         _print_indent('')
@@ -347,7 +375,7 @@ class CLIHelp(object):
             separator=FIRST_LINE_PREFIX if help_file.short_summary else '',
             summary=help_file.short_summary if help_file.short_summary else ''
         )
-        _print_indent(line, indent)
+        _print_indent(line, indent, width=self.textwrap_width)
 
         def _build_long_summary(item):
             lines = []
@@ -355,11 +383,13 @@ class CLIHelp(object):
                 lines.append(item.long_summary)
             if item.deprecate_info:
                 lines.append(str(item.deprecate_info.message))
+            if item.preview_info:
+                lines.append(str(item.preview_info.message))
             return ' '.join(lines)
 
         indent += 1
         long_sum = _build_long_summary(help_file)
-        _print_indent(long_sum, indent)
+        _print_indent(long_sum, indent, width=self.textwrap_width)
 
     def _print_groups(self, help_file):
 
@@ -369,15 +399,18 @@ class CLIHelp(object):
         self.max_line_len = 0
 
         def _build_tags_string(item):
-            PREVIEW_TAG, PREVIEW_TAG_LEN = _get_preview_tag()
+
+            preview_info = getattr(item, 'preview_info', None)
+            preview = preview_info.tag if preview_info else ''
+
             deprecate_info = getattr(item, 'deprecate_info', None)
             deprecated = deprecate_info.tag if deprecate_info else ''
-            preview = PREVIEW_TAG if getattr(item, 'preview_info', None) else ''
+
             required = REQUIRED_TAG if getattr(item, 'required', None) else ''
-            tags = ' '.join([x for x in [str(deprecated), preview, required] if x])
+            tags = ' '.join([x for x in [str(deprecated), str(preview), required] if x])
             tags_len = sum([
                 len(deprecated),
-                PREVIEW_TAG_LEN if preview else 0,
+                len(preview),
                 len(required),
                 tags.count(' ')
             ])
@@ -407,7 +440,12 @@ class CLIHelp(object):
         def _print_items(layouts):
             for layout in layouts:
                 layout['padding'] = ' ' * _get_padding_len(self.max_line_len, layout)
-                _print_indent(LINE_FORMAT.format(**layout), indent, _get_hanging_indent(self.max_line_len, indent))
+                _print_indent(
+                    LINE_FORMAT.format(**layout),
+                    indent,
+                    _get_hanging_indent(self.max_line_len, indent),
+                    width=self.textwrap_width,
+                )
             _print_indent('')
 
         groups = [c for c in help_file.children if isinstance(c, self.group_help_cls)]
@@ -426,13 +464,13 @@ class CLIHelp(object):
 
     @staticmethod
     def _get_choices_defaults_sources_str(p):
-        choice_str = u'  Allowed values: {0}.'.format(', '.join(sorted([str(x) for x in p.choices]))) \
+        choice_str = u'  Allowed values: {}.'.format(', '.join(sorted([str(x) for x in p.choices]))) \
             if p.choices else ''
-        default_str = u'  Default: {0}.'.format(p.default) \
+        default_str = u'  Default: {}.'.format(p.default) \
             if p.default and p.default != argparse.SUPPRESS else ''
-        value_sources_str = u'  Values from: {0}.'.format(', '.join(p.value_sources)) \
+        value_sources_str = u'  Values from: {}.'.format(', '.join(p.value_sources)) \
             if p.value_sources else ''
-        return u'{0}{1}{2}'.format(choice_str, default_str, value_sources_str)
+        return u'{}{}{}'.format(choice_str, default_str, value_sources_str)
 
     @staticmethod
     def print_description_list(help_files):
@@ -440,11 +478,11 @@ class CLIHelp(object):
         max_length = max(len(f.name) for f in help_files) if help_files else 0
         for help_file in sorted(help_files, key=lambda h: h.name):
             column_indent = max_length - len(help_file.name)
-            _print_indent(u'{0}{1}{2}'.format(help_file.name,
-                                              ' ' * column_indent,
-                                              FIRST_LINE_PREFIX + help_file.short_summary
-                                              if help_file.short_summary
-                                              else ''),
+            _print_indent(u'{}{}{}'.format(help_file.name,
+                                           ' ' * column_indent,
+                                           FIRST_LINE_PREFIX + help_file.short_summary
+                                           if help_file.short_summary
+                                           else ''),
                           indent,
                           _get_hanging_indent(max_length, indent))
 
@@ -459,7 +497,7 @@ class CLIHelp(object):
             _print_indent(u'{0}'.format(e.text), indent)
             print('')
 
-    def _print_arguments(self, help_file):
+    def _print_arguments(self, help_file):  # pylint: disable=too-many-statements
 
         LINE_FORMAT = u'{name}{padding}{tags}{separator}{short_summary}'
         indent = 1
@@ -471,15 +509,18 @@ class CLIHelp(object):
             return None
 
         def _build_tags_string(item):
-            PREVIEW_TAG, PREVIEW_TAG_LEN = _get_preview_tag()
+
+            preview_info = getattr(item, 'preview_info', None)
+            preview = preview_info.tag if preview_info else ''
+
             deprecate_info = getattr(item, 'deprecate_info', None)
             deprecated = deprecate_info.tag if deprecate_info else ''
-            preview = PREVIEW_TAG if getattr(item, 'preview_info', None) else ''
+
             required = REQUIRED_TAG if getattr(item, 'required', None) else ''
-            tags = ' '.join([x for x in [str(deprecated), preview, required] if x])
+            tags = ' '.join([x for x in [str(deprecated), str(preview), required] if x])
             tags_len = sum([
                 len(deprecated),
-                PREVIEW_TAG_LEN if preview else 0,
+                len(preview),
                 len(required),
                 tags.count(' ')
             ])
@@ -526,12 +567,17 @@ class CLIHelp(object):
                     last_group_name = layout['group_name']
 
                 layout['padding'] = ' ' * _get_padding_len(self.max_line_len, layout)
-                _print_indent(LINE_FORMAT.format(**layout), indent, _get_hanging_indent(self.max_line_len, indent))
+                _print_indent(
+                    LINE_FORMAT.format(**layout),
+                    indent,
+                    _get_hanging_indent(self.max_line_len, indent),
+                    width=self.textwrap_width,
+                )
 
                 indent = 2
                 long_summary = layout.get('long_summary', None)
                 if long_summary:
-                    _print_indent(long_summary, indent)
+                    _print_indent(long_summary, indent, width=self.textwrap_width)
 
             _print_indent('')
 
@@ -551,6 +597,9 @@ class CLIHelp(object):
             deprecate_info = getattr(item, 'deprecate_info', None)
             if deprecate_info:
                 lines.append(str(item.deprecate_info.message))
+            preview_info = getattr(item, 'preview_info', None)
+            if preview_info:
+                lines.append(str(item.preview_info.message))
             return ' '.join(lines)
 
         group_registry = ArgumentGroupRegistry([p.group_name for p in help_file.parameters if p.group_name])
@@ -570,6 +619,14 @@ class CLIHelp(object):
         if help_file.long_summary or getattr(help_file, 'deprecate_info', None):
             _print_indent('')
 
+        # fix incorrect groupings instead of crashing
+        if help_file.type == 'command' and not isinstance(help_file, CommandHelpFile):
+            help_file.type = 'group'
+            logger.info("'%s' is labeled a command but is actually a group!", help_file.delimiters)
+        elif help_file.type == 'group' and not isinstance(help_file, GroupHelpFile):
+            help_file.type = 'command'
+            logger.info("'%s' is labeled a group but is actually a command!", help_file.delimiters)
+
         if help_file.type == 'command':
             _print_indent('Arguments')
             self._print_arguments(help_file)
@@ -580,7 +637,7 @@ class CLIHelp(object):
 
     def __init__(self, cli_ctx=None, privacy_statement='', welcome_message='',
                  group_help_cls=GroupHelpFile, command_help_cls=CommandHelpFile,
-                 help_cls=HelpFile):
+                 help_cls=HelpFile, textwrap_width=100):
         """ Manages the generation and production of help in the CLI
 
         :param cli_ctx: CLI Context
@@ -595,6 +652,8 @@ class CLIHelp(object):
         :type command_help_cls: HelpFile
         :param command_help_cls: Class to use for formatting generic help.
         :type command_help_cls: HelpFile
+        :param textwrap_width: Line length to which text will be wrapped.
+        :type textwrap_width: int
         """
         from .cli import CLI
         if cli_ctx is not None and not isinstance(cli_ctx, CLI):
@@ -606,6 +665,7 @@ class CLIHelp(object):
         self.group_help_cls = group_help_cls
         self.command_help_cls = command_help_cls
         self.help_cls = help_cls
+        self.textwrap_width = textwrap_width
 
     def show_privacy_statement(self):
         ran_before = self.cli_ctx.config.getboolean('core', 'first_run', fallback=False)
@@ -615,7 +675,7 @@ class CLIHelp(object):
             self.cli_ctx.config.set_value('core', 'first_run', 'yes')
 
     def show_welcome_message(self):
-        _print_indent(self.welcome_message)
+        _print_indent(self.welcome_message, width=self.textwrap_width)
 
     def show_welcome(self, parser):
         self.show_privacy_statement()
