@@ -10,6 +10,7 @@ import sys
 from collections import defaultdict
 
 from .deprecation import ImplicitDeprecated, resolve_deprecate_info
+from .preview import ImplicitPreviewItem, resolve_preview_info
 from .util import CLIError, CtxTypeError, CommandResultItem, todict
 from .parser import CLICommandParser
 from .commands import CLICommandsLoader
@@ -57,24 +58,35 @@ class CommandInvoker(object):
     def _filter_params(self, args):  # pylint: disable=no-self-use
         # Consider - we are using any args that start with an underscore (_) as 'private'
         # arguments and remove them from the arguments that we pass to the actual function.
-        params = dict([(key, value)
-                       for key, value in args.__dict__.items()
-                       if not key.startswith('_')])
+        params = {key: value
+                  for key, value in args.__dict__.items()
+                  if not key.startswith('_')}
         params.pop('func', None)
         params.pop('command', None)
         return params
 
-    def _rudimentary_get_command(self, args):  # pylint: disable=no-self-use
+    def _rudimentary_get_command(self, args):
         """ Rudimentary parsing to get the command """
         nouns = []
-        for i, current in enumerate(args):
-            try:
-                if current[0] == '-':
-                    break
-            except IndexError:
-                pass
-            args[i] = current.lower()
-            nouns.append(args[i])
+        command_names = self.commands_loader.command_table.keys()
+        for arg in args:
+            if arg and arg[0] != '-':
+                nouns.append(arg)
+            else:
+                break
+
+        def _find_args(args):
+            search = ' '.join(args).lower()
+            return next((x for x in command_names if x.startswith(search)), False)
+
+        # since the command name may be immediately followed by a positional arg, strip those off
+        while nouns and not _find_args(nouns):
+            del nouns[-1]
+
+        # ensure the command string is case-insensitive
+        for i in range(len(nouns)):
+            args[i] = args[i].lower()
+
         return ' '.join(nouns)
 
     def _validate_cmd_level(self, ns, cmd_validator):  # pylint: disable=no-self-use
@@ -106,6 +118,7 @@ class CommandInvoker(object):
             err = sys.exc_info()[1]
             getattr(parsed_ns, '_parser', self.parser).validation_error(str(err))
 
+    # pylint: disable=too-many-statements
     def execute(self, args):
         """ Executes the command invocation
 
@@ -119,16 +132,19 @@ class CommandInvoker(object):
         self.cli_ctx.raise_event(EVENT_INVOKER_PRE_CMD_TBL_CREATE, args=args)
         cmd_tbl = self.commands_loader.load_command_table(args)
         command = self._rudimentary_get_command(args)
+        self.cli_ctx.invocation.data['command_string'] = command
         self.commands_loader.load_arguments(command)
 
         self.cli_ctx.raise_event(EVENT_INVOKER_POST_CMD_TBL_CREATE, cmd_tbl=cmd_tbl)
         self.parser.load_command_table(self.commands_loader)
         self.cli_ctx.raise_event(EVENT_INVOKER_CMD_TBL_LOADED, parser=self.parser)
-        if not args:
+
+        arg_check = [a for a in args if a not in ['--verbose', '--debug']]
+        if not arg_check:
             self.cli_ctx.completion.enable_autocomplete(self.parser)
             subparser = self.parser.subparsers[tuple()]
             self.help.show_welcome(subparser)
-            return None
+            return CommandResultItem(None, exit_code=0)
 
         if args[0].lower() == 'help':
             args[0] = '--help'
@@ -141,15 +157,21 @@ class CommandInvoker(object):
 
         self._validation(parsed_args)
 
+        # save the command name (leaf in the tree)
         self.data['command'] = parsed_args.command
-
-        params = self._filter_params(parsed_args)
-
         cmd = parsed_args.func
+        if hasattr(parsed_args, 'cmd'):
+            parsed_args.cmd = cmd
         deprecations = getattr(parsed_args, '_argument_deprecations', [])
         if cmd.deprecate_info:
             deprecations.append(cmd.deprecate_info)
 
+        previews = getattr(parsed_args, '_argument_previews', [])
+        if cmd.preview_info:
+            previews.append(cmd.preview_info)
+
+        params = self._filter_params(parsed_args)
+
         # search for implicit deprecation
         path_comps = cmd.name.split()[:-1]
         implicit_deprecate_info = None
@@ -164,9 +186,23 @@ class CommandInvoker(object):
             del deprecate_kwargs['_get_message']
             deprecations.append(ImplicitDeprecated(**deprecate_kwargs))
 
+        # search for implicit preview
+        path_comps = cmd.name.split()[:-1]
+        implicit_preview_info = None
+        while path_comps and not implicit_preview_info:
+            implicit_preview_info = resolve_preview_info(self.cli_ctx, ' '.join(path_comps))
+            del path_comps[-1]
+
+        if implicit_preview_info:
+            preview_kwargs = implicit_preview_info.__dict__.copy()
+            preview_kwargs['object_type'] = 'command'
+            previews.append(ImplicitPreviewItem(**preview_kwargs))
+
         colorama.init()
         for d in deprecations:
             print(d.message, file=sys.stderr)
+        for p in previews:
+            print(p.message, file=sys.stderr)
         colorama.deinit()
 
         cmd_result = parsed_args.func(params)
@@ -177,5 +213,6 @@ class CommandInvoker(object):
         self.cli_ctx.raise_event(EVENT_INVOKER_FILTER_RESULT, event_data=event_data)
 
         return CommandResultItem(event_data['result'],
+                                 exit_code=0,
                                  table_transformer=cmd_tbl[parsed_args.command].table_transformer,
                                  is_query_active=self.data['query_active'])
