@@ -36,7 +36,7 @@ def format_json(obj):
     result = obj.result
     # OrderedDict.__dict__ is always '{}', to persist the data, convert to dict first.
     input_dict = dict(result) if hasattr(result, '__dict__') else result
-    return json.dumps(input_dict, indent=2, sort_keys=True, cls=_ComplexEncoder,
+    return json.dumps(input_dict, ensure_ascii=False, indent=2, sort_keys=True, cls=_ComplexEncoder,
                       separators=(',', ': ')) + '\n'
 
 
@@ -96,8 +96,6 @@ class OutputProducer(object):
         args = kwargs.get('args')
         # Set the output type for this invocation
         cli_ctx.invocation.data['output'] = getattr(args, OutputProducer.ARG_DEST)
-        # We've handled the argument so remove it
-        delattr(args, OutputProducer.ARG_DEST)
 
     def __init__(self, cli_ctx=None):
         """ Manages the production of output from the result of a command invocation
