@@ -3,12 +3,16 @@
 # Licensed under the MIT License. See License.txt in the project root for license information.
 # --------------------------------------------------------------------------------------------
 
+from __future__ import print_function
+
 import argparse
 
 from .deprecation import Deprecated
 from .events import EVENT_PARSER_GLOBAL_CREATE
+from .log import get_logger
 from .util import CtxTypeError
 
+logger = get_logger(__name__)
 
 # List of keyword arguments supported in argparse
 # from https://github.com/python/cpython/blob/master/Lib/argparse.py#L748
@@ -40,23 +44,50 @@ class CLICommandParser(argparse.ArgumentParser):
     def _add_argument(obj, arg):
         """ Only pass valid argparse kwargs to argparse.ArgumentParser.add_argument """
         argparse_options = {name: value for name, value in arg.options.items() if name in ARGPARSE_SUPPORTED_KWARGS}
-        scrubbed_options_list = []
-        for item in arg.options_list:
-            if isinstance(item, Deprecated):
-                # don't add expired options to the parser
-                if item.expired():
-                    continue
+        if arg.options_list:
+            scrubbed_options_list = []
+            for item in arg.options_list:
 
-                class _DeprecatedOption(str):
-                    def __new__(cls, *args, **kwargs):
-                        instance = str.__new__(cls, *args, **kwargs)
-                        return instance
+                if isinstance(item, Deprecated):
+                    # don't add expired options to the parser
+                    if item.expired():
+                        continue
 
-                option = _DeprecatedOption(item.target)
-                setattr(option, 'deprecate_info', item)
-                item = option
-            scrubbed_options_list.append(item)
-        return obj.add_argument(*scrubbed_options_list, **argparse_options)
+                    class _DeprecatedOption(str):
+                        def __new__(cls, *args, **kwargs):
+                            instance = str.__new__(cls, *args, **kwargs)
+                            return instance
+
+                    option = _DeprecatedOption(item.target)
+                    setattr(option, 'deprecate_info', item)
+                    item = option
+                scrubbed_options_list.append(item)
+            return obj.add_argument(*scrubbed_options_list, **argparse_options)
+
+        if 'required' in argparse_options:
+            del argparse_options['required']
+        if 'metavar' not in argparse_options:
+            argparse_options['metavar'] = '<{}>'.format(argparse_options['dest'].upper())
+        return obj.add_argument(**argparse_options)
+
+    @staticmethod
+    def _expand_prefixed_files(args):
+        """ Load arguments prefixed with '@' from file as string
+
+        :param args: Arguments passed from command line
+        :type args: list
+        """
+        for arg, _ in enumerate(args):
+            if args[arg].startswith('@'):
+                try:
+                    logger.debug('Attempting to read file %s', args[arg][1:])
+                    with open(args[arg][1:], 'r') as f:
+                        content = f.read()
+                    args[arg] = content
+                except IOError:
+                    # Leave arg unmodified
+                    logger.debug('File Error: Failed to open %s, assume not a file', args[arg][1:])
+        return args
 
     def __init__(self, cli_ctx=None, cli_help=None, **kwargs):
         """ Create the argument parser
@@ -141,6 +172,7 @@ class CLICommandParser(argparse.ArgumentParser):
                     param = CLICommandParser._add_argument(command_parser, arg)
                 param.completer = arg.completer
                 param.deprecate_info = arg.deprecate_info
+                param.preview_info = arg.preview_info
             command_parser.set_defaults(
                 func=metadata,
                 command=command_name,
@@ -216,3 +248,36 @@ class CLICommandParser(argparse.ArgumentParser):
                                 self._actions[-1] if is_group else self,
                                 is_group)
         self.exit()
+
+    def parse_args(self, args=None, namespace=None):
+        """ Overrides argparse.ArgumentParser.parse_args
+
+        Enables '@'-prefixed files to be expanded before arguments are processed
+        by ArgumentParser.parse_args as usual
+        """
+        self._expand_prefixed_files(args)
+        return super(CLICommandParser, self).parse_args(args)
+
+    def _check_value(self, action, value):
+        # Override to customize the error message when a argument is not among the available choices
+        # converted value must be one of the choices (if specified)
+        import difflib
+        import sys
+
+        if action.choices is not None and value not in action.choices:
+            # parser has no `command_source`, value is part of command itself
+            error_msg = "{prog}: '{value}' is not in the '{prog}' command group. See '{prog} --help'.".format(
+                prog=self.prog, value=value)
+            logger.error(error_msg)
+            candidates = difflib.get_close_matches(value, action.choices, cutoff=0.7)
+            if candidates:
+                print_args = {
+                    's': 's' if len(candidates) > 1 else '',
+                    'verb': 'are' if len(candidates) > 1 else 'is',
+                    'value': value
+                }
+                suggestion_msg = "\nThe most similar choice{s} to '{value}' {verb}:\n".format(**print_args)
+                suggestion_msg += '\n'.join(['\t' + candidate for candidate in candidates])
+                print(suggestion_msg, file=sys.stderr)
+
+            self.exit(2)
